// ****************************************************
// User Defined File Loader Functions
// J. J. Weimer
// History: $Revision: 512 $ on $Date: 2008-02-11 11:59:35 -0500 (Mon, 11 Feb 2008) $

#pragma rtGlobals = 1
#pragma IgorVersion=6.03
#pragma version = 1.10

#include "udFLStandardStructure", version >= 1.10

// ****************************************************

Function udFLZeroStructure(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFL.mimetype = ""
	udFL.extensions = ""
	udFL.itemsType = 0
	udFL.itemsDim[0] = 0
	udFL.itemsDim[1] = 0
	udFL.itemsDim[2] = 0
	udFL.itemsDim[3] = 0
	udFL.procModes = "Auto;"
	udFL.setNames = 0
	udFL.eventCode = 0
	udFL.userCtrl = 0
	udFL.userData = ""
	udFL.reportCtrl = 0
	udFL.plugIns = FunctionList("udFLPlugIn*",";","KIND:6")
	udFL.namePrefix = ""
	udFL.pathStr = ""
	udFL.fileList = ""
	udFL.returnCtrl = 0
	udFL.toFolder = "root:"
	udFL.itemsCount = 0
	udFL.itLst.GLOBALS = ""
	udFL.itLst.LISTS = ""
	udFL.itLst.WAVES = ""
	udFL.itLst.MATRICES = ""
	udFL.itLst.IMAGES = ""
	udFL.itLst.NOTEBOOKS = ""
	udFL.itLst.PROCEDURES = ""
	udFL.errCode = 0
	udFL.errMsg = ""
	
	return 0
	
end

// ****************************************************
// Igor Pro User Defined File Loader Assure File Paths

Function udFLAssureFilePaths(udFL)
	STRUCT udFLStandardStructure &udFL
	
	variable ic, nc = ItemsInList(udFL.fileList), fpsame = 1
	variable nsf=0, nsg=0
	string pnameo, pnamef, nflist = ""
	
	pnameo = ParseFilePath(1,StringFromList(0,udFL.fileList),":",1,0)
	
	// no path, just file list
	
	if (strlen(udFL.pathStr)==0)
	
		// one file in list
		
		if (nc==1)
		
			// no path is with file name (error) or a path is with file name
			
			if (strlen(pnameo)==0)
				udFLSetError(udFL,"No valid path string found",-1)
				return -1
			else
				udFL.fileList = ParseFilePath(0,StringFromList(0,udFL.fileList),":",1,0) + ";"
				udFL.pathStr = pnameo
				return nc
			endif
			
		// more than one file in list
		
		else
			
			for (ic=0;ic<nc;ic+=1)
				pnamef = ParseFilePath(1,StringFromList(ic,udFL.fileList),":",1,0)
		
				// confirm all files have paths (otherwise error because inconsistent paths)
				
				if (strlen(pnamef)==0)
					return -1
				endif
				
				// test whether all paths are the same
				// collect file list without paths
				
				if (cmpstr(pnamef,pnameo)!=0)
					fpsame = 0
				else
					nflist += ParseFilePath(0,StringFromList(ic,udFL.fileList),":",1,0) + ";"
				endif
			endfor
		endif
	
	// path string and filelist string
	
	else
	
		// one file in list
		
		if (nc==1)
		
			// no path on file name
			
			if (strlen(pnameo)==0)
				return nc
			else
			
				// path name on file is not the same as pathStr
				// otherwise strip off path string from file name
				
				if (cmpstr(pnameo,udFL.pathStr)!=0)
					udFLSetError(udFL,"Path string on file and in path are inconsistent",-1)
					return -1
				else
					udFL.fileList = ParseFilePath(0,StringFromList(0,udFL.fileList),":",1,0) + ";"
					return nc
				endif
			endif
			
		// more than one file in list
		
		else
		
			// how many files have paths?
			
			for (ic=0;ic<nc;ic+=1)
				pnamef = ParseFilePath(1,StringFromList(ic,udFL.fileList),":",1,0)
				if (strlen(pnamef)!=0)
					nsf +=1
				endif
			endfor
			
			// NONE!
			
			if (nsf==0)
				return nc
			endif
			
			// how many files have paths different from pathStr?
			
			for (ic=0;ic<nc;ic+=1)
				pnamef = ParseFilePath(1,StringFromList(ic,udFL.fileList),":",1,0)
				if (strlen(pnamef)!=0)
					if (cmpstr(pnamef,udFL.pathStr)!=0)
						nsg +=1
					endif
				endif
				nflist += ParseFilePath(0,StringFromList(ic,udFL.fileList),":",1,0) + ";"
			endfor
			
			// SOME!
			
			if (nsg!=0)
				
				//inconsistent mixture of files with and without paths
				// or all files have paths (OK even though they are different from pathStr)
				
				if (nsf!=nc)
					udFLSetError(udFL,"Inconsistent mixture of files with/without path string",-1)
					return -1
				else
					fpsame = 0
				endif			
			endif
			
		endif
				
	endif
	
	// conclusion for more than one file in list
	
	if (fpsame)
		udFL.fileList = nflist
		udFL.pathStr = pnameo
	else
		udFL.pathStr = ""
	endif
	
	return nc
end

// ****************************************************
// Igor Pro User Defined File Loader Set Data Folder

Function udFLSetDataFolder(udFL,[setit])
	STRUCT udFLStandardStructure &udFL
	variable setit
	
	if (strlen(udFL.toFolder)==0)
		return -1
	endif
	
	string subfolder
	variable ic = 0
	
	string cdf = GetDataFolder(1)
	SetDataFolder root:
	
	do
		subfolder = StringFromList(ic,udFL.toFolder,":")
		if (strlen(subfolder) == 0)
			break
		endif
		NewDataFolder/O/S $subfolder
		ic += 1
	while(1)
	
	if (!ParamIsDefault(setit))
		if (!setit)
			SetDataFolder $cdf
		endif
	endif
	
	return 0
	
end

// ****************************************************
// Igor Pro User Defined File Loader Set Error
//

Function udFLSetError(udFL,errStr,errCode)
	STRUCT udFLStandardStructure &udFL
	string errStr
	variable errCode
	
	udFL.errMsg = errStr
	udFL.errCode = errCode
	return 0
end

// ****************************************************
// Igor Pro User Defined File Loader Handle Error
//

Function udFLHandleError(udFL)
	STRUCT udFLStandardStructure &udFL

	if (udFL.errCode<0)
		DoAlert 0, ("ABORTING: " + udFL.errMsg)
		Abort
	else
		if (strlen(udFL.errMsg)!=0)
			print "WARNING: ", udFL.errMsg
		endif
		return 0
	endif
end

// ****************************************************
// Igor Pro User Defined File Loader Query proc Modes

Function/S udFLQueryprocModes(fp)
	string fp

	STRUCT udFLStandardStructure udFL
	
	FUNCREF udFLProtoFunc f = $fp
	udFL.userCtrl = 0
	udFL.procModes = ""
	f(udFL)
	return (udFL.procModes)
end

// ****************************************************
// Igor Pro User Defined File Loader Query setNames

Function/S udFLQuerysetNames(fp)
	string fp

	STRUCT udFLStandardStructure udFL
	string sNL = "Auto;"
	
	FUNCREF udFLProtoFunc f = $fp
	udFL.userCtrl = 0
	udFL.setNames = 0
	f(udFL)
	
	if (udFL.setNames == 1)
		sNL += "\\M1(Filename+Auto#;\\M1(File Header Info;\\M1(Prefix+Auto#;\\M1(Manually;\\M1(Loader-Defined;"
		return (sNL)
	endif
	if (udFL.setNames == 31)
		sNL +=  "Filename+Auto#;File Header Info;Prefix+Auto#;Manually;\\M1(Loader-Defined;"
		return (sNL)
	endif
	if ((udFL.setNames & 2^1) != 0)
		sNL += "Filename+Auto#;"
	else
		sNL += "\\M1(Filename+Auto#;"	
	endif
	if ((udFL.setNames & 2^2) != 0)
		sNL += "File Header Info;"
	else
		sNL += "\\M1(File Header Info;"	
	endif
	if ((udFL.setNames & 2^3) != 0)
		sNL += "Prefix+Auto#;"
	else
		sNL += "\\M1(Prefix+Auto#;"	
	endif
	if ((udFL.setNames & 2^4) != 0)
		sNL += "Manually;"
	else
		sNL += "\\M1(Manually;"	
	endif
	if ((udFL.setNames & 2^5) != 0)
		sNL += udFL.userData
	else
		sNL += "\\M1(Loader-Defined;"	
	endif	
	return (sNL)
end

// ****************************************************
// Igor Pro User Defined File Loader Initialize Loader

Function udFLInitializeLoader(udFL,fp)
	STRUCT udFLStandardStructure &udFL
	string fp
	
	FUNCREF udFLProtoFunc f = $fp
	udFL.userCtrl=-1
	return (f(udFL))
end

// ****************************************************
// Igor Pro User Defined File Loader Run Loader

Function udFLRunLoader(udFL,fp)
	STRUCT udFLStandardStructure &udFL
	string fp
	
	FUNCREF udFLProtoFunc f = $fp
	return (f(udFL))
end

// the proto function that defines the loader function format

Function udFLProtoFunc(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFL.userData = "no data yet"
	udFL.procModes = "Auto;Others TBD;"
end

// ****************************************************
// Igor Pro User Defined File Loader StripUserDataString

Function/S udFLStripUserDataString(udFL)
	STRUCT udFLStandardStructure &udFL
	
	string fll = StringFromList(0,udFL.UserData)
	if (strlen(fll)!=0)
		udFL.UserData = RemoveListItem(0,udFL.UserData)
	endif
	
	return (fll)
end

// ****************************************************
// Igor Pro User Defined File Loader Check for PlugIn

Function udFLCheckForPlugIn(piname)
	string piname
	
	string fll = FunctionList("udFLPlugIn*",";","KIND:6")
	return ( cmpstr(ReplaceString("udFLPlugIn",fll,""),piname) )
end

// ****************************************************
// Igor Pro User Defined File Loader Loader List

Function/S udFLLoaderList(how)
	variable how
	
	string fll = FunctionList("udStFiLr*",";","KIND:6")
	if (strlen(fll)==0)
		return "None Available"
	endif
	if (how)
		return (fll)
	else
		return (ReplaceString("udStFiLr",fll,""))
	endif
end

// ****************************************************
// Igor Pro User Defined File Loader Rename Input

Function/S udFLRenameInput(cList,prefix)
	string cList, prefix
	
	string nList = "", iName ="", xList="", nname = ""
	
	variable ic, nt = ItemsInList(cList), oT, nx=0
	
	for (ic=0;ic<nt;ic+=1)
		iName = StringFromList(ic,cList)
		oT = exists(iName)
		switch(oT)
			case 1:	// waves
				xList = udFLDataFolderDir(2,prefix)
				if (strlen(xList) == 0)
					nname = prefix + num2str(ic)
				else
					nname = prefix + num2str(ItemsInList(xList))
				endif
				if (WaveType(iName)==0)
					wave/T twave = $iName
					rename twave $nname
				else
					wave nwave = $iName
					rename nwave $nname
				endif
				nList += nname + ";"
				break
			case 2:	// strings or variables
				NVAR tv = $iName
				if (NVAR_exists(tv))
					xList = udFLDataFolderDir(4,("V_" + prefix))
					if (strlen(xList) == 0)
						nname = "V_" + prefix + num2str(ic)
					else
						nname = "V_" + prefix + num2str(ItemsInList(xList))
					endif
					rename tv $nname
					break
				endif
				SVAR sv = $iName
				if (SVAR_exists(sv))
					xList = udFLDataFolderDir(4,("S_" + prefix))
					if (strlen(xList) == 0)
						nname = "S_" + prefix + num2str(ic)
					else
						nname = "S_" + prefix + num2str(ItemsInList(xList))
					endif
					SVAR sv = $iName
					rename sv $nname
				endif
				nList += nname + ";"
				break
		endswitch
	endfor
	
	return nList
end

Static Function/S udFLDataFolderDir(mode,prefix)
	variable mode
	string prefix
	
	string theDirList = DataFolderDir(mode)
	string tmpList="", regStr = ("^" + prefix + ".")
	
	switch(mode)
		case 2:
			tmpList = ReplaceString("WAVES:",theDirList,"")
			break
		case 4:
			tmpList = ReplaceString("VARIABLES:",theDirList,"")
			break
		case 8:
			tmpList = ReplaceString("STRINGS:",theDirList,"")
			break
	endswitch
	tmpList = ReplaceString(",",tmpList,";")
	tmpList = SortList(tmpList,";",16)
	tmpList = GrepList(tmpList,regStr,0,";")
	
	return tmpList
end

// ****************************************************
// Folder List
// returns list of topmost folders from root with prepend before it

Function/S udFLFolderList()

	string dflist, objName=""
	variable ic = 0
	
	dflist = "root;"
	
	do
		objName = GetIndexedObjName("root:", 4, ic)
		if (strlen(objName) == 0)
			break
		else
			if (cmpstr(objName,"Packages")!=0)
				dflist += objName + ";"
			endif
		endif
		ic += 1
	while(1)
	
	return dflist
end

