//Standard Loader Panel Demo
//
// This shows how standardizing a user defined
// file loader can enable "plug-and-load" operation
//
// Create a file loader by conventions
// It should appear in the panel list for "testing"

#pragma rtGlobals=1

#pragma version=1.05
#pragma IgorVersion=6.0

#include "udFLStandardStructure", version = 1.06
#include "udFLStandardFunctions", version = 1.06

Menu "Macros"
	"Standard Loader Demo",MakeLoaderPanel()
End

// make the loader panel

Function MakeLoaderPanel()

	InitSimpleStandardLoaderPanel()

	NewPanel/K=1/W=(381,55,763,264)/N=udFLPanel
	
	PopupMenu popup0,pos={21,18},size={115,20},title="File Loader",fSize=12, proc=udChangeFileLoader
	PopupMenu popup0,mode=1,value=#"udFLLoaderList(0)"

	Button button0,pos={200,166},size={50,20},title="Test",fSize=14, proc=TestudFL

	Button button1,pos={280,166},size={50,20},title="Load",fSize=14, proc=LoadudFL

	SetVariable setvar0,pos={23,136},size={300,18},title="Store @ root:",fSize=12
	SetVariable setvar0,value= root:Packages:SimpleFileLoaderDemo:toFolder

	PopupMenu popup1,pos={23,75},size={80,20},title="Mode",fSize=12
	PopupMenu popup1,mode=1,value= #"udGetFileLoaderModeList()"

	SetVariable setvar1,pos={23,108},size={250,18},title="User Data",fSize=12
	SetVariable setvar1,value= root:Packages:SimpleFileLoaderDemo:userData

	SetVariable setvar2,pos={23,44},size={300,18},title="File List",fSize=12
	SetVariable setvar2,value= root:Packages:SimpleFileLoaderDemo:fileList
	
	return 0

end

// initialize panel package

Static Function InitSimpleStandardLoaderPanel()

	string cdf = GetDataFolder(1)
	
	SetDataFolder root:
	NewDataFolder/O/S Packages
	NewDataFolder/O/S SimpleFileLoaderDemo
	string/G fileList=""
	string/G userData=""
	string/G toFolder=""

	SetDataFolder $cdf
	
	return 0
	
end

// get the list of loading modes for a given loader function

Function/S udGetFileLoaderModeList()

	ControlInfo popup0
	variable mode = V_value
	
	string loaderFunc = StringFromList(mode-1,udFLLoaderList(1))

	if (strlen(loaderFunc)==0)
		loaderFunc = "udFLProtoFunct"
	endif
		
	return (udFLQueryprocModes(loaderFunc))

end

// Controls

// change things if the file loader function is changed

Function udChangeFileLoader(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	switch(pa.eventCode)
		case 2:
			
			string loaderFunc = StringFromList(pa.popNum-1,udFLLoaderList(0))			
			if (cmpstr(loaderFunc,pa.popStr)!=0)
				return 0
			endif
			PopupMenu popup1, mode=1
			break
	endswitch

	return 0
End

// the "test" button

Function TestudFL(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	STRUCT udFLStandardStructure udFL

	SVAR fileList = root:Packages:SimpleFileLoaderDemo:fileList
	SVAR userData = root:Packages:SimpleFileLoaderDemo:userData
	SVAR toFolder = root:Packages:SimpleFileLoaderDemo:toFolder
	
	switch( ba.eventCode )
		case 2: // mouse up
			ControlInfo popup0
			print "\r---\rUse loader ... ", S_value
			string loaderFunc = StringFromList(V_value-1,udFLLoaderList(1))			
			udFLInitializeLoader(udFL, loaderFunc)
			udFL.userData = userData
			udFL.toFolder = toFolder
			udFL.fileList = fileList
			udFLAssureFilePaths(udFL)
			ControlInfo popup1
			udFL.userCtrl = V_value
			print udFL
			break
	endswitch

	return 0
End

// the "load" button

Function LoadudFL(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	STRUCT udFLStandardStructure udFL

	SVAR fileList = root:Packages:SimpleFileLoaderDemo:fileList
	SVAR userData = root:Packages:SimpleFileLoaderDemo:userData
	SVAR toFolder = root:Packages:SimpleFileLoaderDemo:toFolder
	
	string loaderFunc
	
	switch( ba.eventCode )
		case 2: // mouse up
			udFL.fileList = fileList
			udFL.pathStr = ""
			if (udFLAssureFilePaths(udFL)<0)
				DoAlert 0, "Improper file list"
				return 0
			endif
			ControlInfo popup0
			loaderFunc = StringFromList(V_value-1,udFLLoaderList(1))
			udFLInitializeLoader(udFL, loaderFunc)
			ControlInfo popup1
			udFL.userCtrl = V_value
			udFL.userData = userData
			udFL.toFolder = toFolder
			if (udFLRunLoader(udFL,loaderFunc)<0)
				if (udFL.errCode<0)
					DoAlert 0, udFL.errMsg
				else
					print udFL.errMsg
				endif
			endif
			break
	endswitch

	return 0
End

// some loader functions with no behavior
// they  return their character regardless of userCtrl

Function udStFiLrTabDelimitedXY(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFL.procModes = "Auto;Delimit;Slice;"
	udFL.mimetype = "TEXT"
	udFL.extensions = ".txt;"
	udFL.itemsType = 1
	udFL.itemsDim[0] = 1
	return 0

end

Function udStFiLrJoesBarbecueGrill(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFL.procModes = "Auto;Mild;Hot;Real Spicy;"
	udFL.mimetype = "BRBQ"
	udFL.extensions = ".bbq;.grl;"
	udFL.itemsType = 2
	udFL.itemsDim[0] = 2	
	return 0

end

// some plug-ins with no behavior

Function udFLPlugInAddSomeSpice(udFL)
	STRUCT udFLStandardStructure &udFL
end

Function udFLPlugInQTXOP(udFL)
	STRUCT udFLStandardStructure &udFL
end
