// Load Panel for Standardized File Loader
// Jeffrey J Weimer
// SVN History: $Revision: 512 $ on $Date: 2008-02-11 11:59:35 -0500 (Mon, 11 Feb 2008) $

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=1.10

#pragma ModuleName=udFLStandardPanel

#include "udFLStandardFunctions", version = 1.10
#include "udFLStandardPanel.lcl"

#include "ScreenSizer", version>=1.10

// load panel structure

Static STRUCTURE udFLPanelStructure

	variable lpw				// panel width
	variable lph				// panel height
	variable spw				// subpanel width
	variable sph				// subpanel height

ENDSTRUCTURE

// initialize load panel width and height (in pixels!!)

Static Function Initialize(sLP)
	STRUCT udFLPanelStructure &sLP
	
	sLP.lpw = 235
	sLP.lph = 330
	sLP.spw = 230
	sLP.sph = 250
	
	ScreenSizer#Initialize(0)
	
	return 0
end

// Show Panel

Function ShowPanel(restart)
	variable restart
	
	// kill or bring forward panel
	
	if (strlen(WinList("udFL_Panel","","WIN:64"))!=0)
		if (restart)
			KillWindow udFL_Panel
		else
			DoWindow/F udFL_Panel
			return 0
		endif
	endif
	
	// (re)initialize globals 
	
	string cdf = GetDataFolder(1)
	
	NewDataFolder/O/S root:Packages
	NewDataFolder/O/S StandardizedFileLoader
	
	string/G loadtofolder="", loadfileprefix="", userData="", loadfunctionname=""
	string/G udFLdatapath="", listofwavesloaded=""
	
	make/O/T/N=0 filesindatapath
	make/O/N=0 nfilesindatapath
	
	SetDataFolder $cdf
	
	// initialize new panel and screen sizes
	
	STRUCT udFLPanelStructure sLP
	Initialize(sLP)
	
	ScreenSizer#MakePanel(-95,5,sLP.lpw,sLP.lph,"Data",other="/K=1/N=udFL_Panel")
	ModifyPanel fixedSize=1, noEdit=1
	
	// Tabs
	
	TabControl tb, pos={1,2},size={sLP.lpw,20}, proc=udFL_tcShowPanelTab
	TabControl tb, tabLabel(0)=udFLpan00, value=0
	
	// Controls
	
	PanelBuildTab(0)
	PanelBuildTab(99)
	
	// finalize new panel
	
	PanelFillTab(0)

	return 0
End

// build a tab on the data panel

Static Function PanelBuildTab(tab)
	variable tab
	
	STRUCT udFLStandardStructure udFL
	udFLZeroStructure(udFL)

	// Control Globals and Mode Settings
		
	SVAR ltf = root:Packages:StandardizedFileLoader:loadtofolder
	SVAR lfp = root:Packages:StandardizedFileLoader:loadfileprefix
	SVAR uD = root:Packages:StandardizedFileLoader:userData
	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname

	// build a specific tab
	
	switch(tab)
	
		case 0:
	
			lfN = StringFromList(0,udFLLoaderList(1))
		
			// Load Tab
			// =====
		
			// Folder List
			
			PopupMenu flist_tb0,pos={20,39},size={122,20},title=udFLpan00a,fSize=12
			PopupMenu flist_tb0, mode=1,popvalue="root", value=#"udFL_FolderList()"
			PopupMenu flist_tb0, proc=udFL_popCreateNewFolderName
		
			// CreateNew Input
		
			SetVariable newdir_tb0,pos={20,65},size={180,17},title="root:",fSize=12
			SetVariable newdir_tb0,value=ltf, proc=udFL_svcValidateLoadFolderName
			SetVariable newdir_tb0, help={"You may NOT use : or space characters here!"}

			// Data Folder Exists Warning

			TitleBox fldrx_tb0,pos={200,65},size={15,15},title="!",fSize=12
			TitleBox fldrx_tb0, help={"This data folder already exists!"}
		
			// FileName
		
			PopupMenu fname_tb0,pos={20,92},size={87,20},title=udFLpan00c,fSize=12
			PopupMenu fname_tb0,mode=1,value=#"udFL_RenameInputList()"
			PopupMenu fname_tb0, proc=udFL_popCreateNewTraceName

			// CreateNew Filename
		
			SetVariable newfname_tb0,pos={20,118},size={180,17},title="input:",fSize=12
			SetVariable newfname_tb0,value=lfp, proc=udFL_svcValidateNewTraceName
			SetVariable newfname_tb0, help={"You may NOT use certain characters here!"}

			// Loader
		
			PopupMenu loader_tb0,pos={20,148},size={87,20},title=udFLpan00d,fSize=12
			PopupMenu loader_tb0,mode=1,value=#"udFLLoaderList(0)"
			PopupMenu loader_tb0, proc=udFL_popChangeLoaderModes
	
			// Processing Modes
			
			PopupMenu lmode_tb0,pos={20,175},size={87,20},title=udFLpan00e,fSize=12
			PopupMenu lmode_tb0, mode=1, value=#"udFL_FileLoaderModeList()"

			// User Data
			
			SetVariable userData_tb0,pos={20,200},size={200,17},title=udFLpan00f,fSize=12
			SetVariable userData_tb0,value=uD
			SetVariable userData_tb0, help={"Each file loader has its own User Data input"}

			// Report Modes
			
			PopupMenu rpmode_tb0,pos={20,223},size={87,20},title=udFLpan00g,fSize=12
			PopupMenu rpmode_tb0, mode=2, value=udFLpan00h

			// SetPaths
			
			Button setpaths_tb0,pos={20,250},size={50,55}, proc=udFL_bcSetDirPaths
			Button setpaths_tb0,fSize=12, fColor=(49151,49151,65535), userdata="0"
			
			PathInfo udFL_DataFiles
			if (V_flag==0)
				Button setpaths_tb0, title=udFLpan00i1
			else
				Button setpaths_tb0, title=udFLpan00i2, userdata="1"
			endif
			break
		
		default:

			// Close Panel When Complete
		
			CheckBox tb_closeme,pos={34,310},size={139,14},title=udFLpan00j
			CheckBox tb_closeme,value= 0
			
			// Do It!
			
			Button tb_doit,pos={124,260},size={70,20},proc=udFL_bcDoIt
			Button tb_doit, font="Geneva",fSize=14, fColor=(49151,49151,65535)
			break
			
	endswitch
	
	return 0
	
End

// fill a tab when it is selected

Function PanelFillTab(tab)
	variable tab
	
	string tStr
	
	tStr = "!*_tb"+num2str(tab)	
	ModifyControlList ControlNameList("",";",tStr) disable=1
	tStr = "*_tb"+num2str(tab)
	ModifyControlList ControlNameList("",";",tStr) disable=0
	ModifyControlList ControlNameList("",";","tb*") disable=0
			
	// disable select controls on given tab
	
	switch(tab)
		case 0:
			Button tb_doit, title=udFLpan00, userData="Load"
			ControlInfo flist_tb0
			if (cmpstr(S_value,"Create New")!=0)
				SetVariable newdir_tb0 disable=1
				TitleBox fldrx_tb0 disable=1
			else
				ControlInfo newdir_tb0
				tStr = "root:" + S_value
				TitleBox fldrx_tb0 disable=(!DataFolderExists(tStr))
				SetVariable newdir_tb0 disable=0
			endif
			ControlInfo fname_tb0
			SetVariable newfname_tb0 disable=((V_Value!=4)||(V_Value!=6))
			ControlInfo flist_tb1
			if (strlen(FunctionList("udFLPanelSetTab1",";",""))!=0)
				Execute/Q "udFLPanelSetTab1(0)"
			endif
			break
		case 1:
			if (strlen(FunctionList("udFLPanelSetTab1",";",""))!=0)
				Execute/Q "udFLPanelSetTab1(1)"
			endif
			break
	endswitch
	
	udFL_SubPanelShow(tab)
	
	return 0

End

// show or kill the file list panel

Function udFL_SubPanelShow(tab)
	variable tab
	
	STRUCT udFLPanelStructure sLP
	
	wave/T flist = root:Packages:StandardizedFileLoader:filesindatapath
	wave slist =root:Packages:StandardizedFileLoader:nfilesindatapath
	
	string lfname=""
		
	if (tab==0)
	
		// switch from something to load
		
		PathInfo udFL_DataFiles
		if (V_flag)
			lfname = GetUserData("","setpaths_tb0","")
			if (cmpstr(lfname,"1")==0)
				ControlInfo loader_tb0
				lfname = StringFromList(V_value-1,udFLLoaderList(1))
				udFL_GetFilesInDataDirPath(lfname)
				Button setpaths_tb0, userData="0"				
			endif
			if (strlen(ChildWindowList("#"))==0)
				Initialize(sLP)
				ScreenSizer#MakeExtPanel(2,0,sLP.spw,sLP.sph,"File List",other="/HOST=#/N=udFL_FileListPanel/K=2")
				ListBox theList,pos={4,4},size={0.9*sLP.spw,0.9*sLP.sph}, mode=4, listwave=flist, selWave=slist
				SetActiveSubWindow ##
			endif
		else
			if (strlen(ChildWindowList("#"))!=0)
				KillWindow #udFL_FileListPanel
				Button setpaths_tb0, title=udFLpan00i1, userData="0"
			endif
		endif
	else
	
		// switch from load to something else
			
		if (strlen(ChildWindowList("#"))!=0)
			KillWindow #udFL_FileListPanel
		endif

	endif
	
	return 0
end

// load data

Function udFL_LoadData()
	
	STRUCT udFLStandardStructure udFL
		
	wave wnfiles=root:Packages:StandardizedFileLoader:nfilesindatapath
	wave/T files=root:Packages:StandardizedFileLoader:filesindatapath
	
	SVAR ltf = root:Packages:StandardizedFileLoader:loadtofolder
	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	SVAR uD = root:Packages:StandardizedFileLoader:userData
	SVAR lfp = root:Packages:StandardizedFileLoader:loadfileprefix
	SVAR llowl = root:Packages:StandardizedFileLoader:listofwavesloaded
	
	// no data path or no files
	
	if (numpnts(wnfiles)==0)
		ControlInfo lmode_tb0
		if (V_value==1)
			DoAlert 0,udFLdb00a
			return 0
		endif
	endif
	
	if (sum(wnfiles)==0)
		ControlInfo lmode_tb0
		if (V_value==1)
			DoAlert 0, udFLdb00b
			return 0
		endif
	endif
		
	string floadname = lFN
	variable ic, nc=numpnts(wnfiles)
	
	// initialize the loader
	
	udFLInitializeLoader(udFL,floadname)
	
	// set the Inputs
	
	ControlInfo lmode_tb0
	udFL.userCtrl = V_value
	udFL.userData = uD
	ControlInfo rpmode_tb0
	udFL.reportCtrl = V_value-2
	ControlInfo fname_tb0
	udFL.setNames = 2^(V_value-1)
	if ((V_value==4)||(V_value==6))
		udFL.namePrefix = lfp
	endif
	
	// set the Localizers

	PathInfo udFL_DataFiles
	udFL.pathStr = S_path
	udFL.returnCtrl = 0
	udFL.toFolder = ltf
	
	for (ic=0;ic<nc;ic+=1)
		if (wnfiles[ic])
			udFL.fileList += files[ic] + ";"
		endif
	endfor
	
	// load the files

	string cdf = GetDataFolder(1)
	
	if (udFL.reportCtrl==1)
		print "\r---\rUsing ... ", floadname
	endif
	
	if (udFLRunLoader(udFL,floadname)<0)
		SetDataFolder $cdf
		udFLHandleError(udFL)
		return -1
	else
		udFLHandleError(udFL)
	endif
	
	nc = udFL.itemsCount
	string theItem
	llowl = ""
	for (ic=0;ic<nc;ic+=1)
		theItem = "root:" + udFL.toFolder + ":" + StringFromList(ic,udFL.itLst.WAVES)
		if (exists(theItem)==1)
			llowl += StringFromList(ic,udFL.itLst.WAVES) + ";"
		endif
	endfor
	
	SetDataFolder $cdf
		
	return 0

End

// CONTROLS

// Tabs

// show a tab when it is clicked

Function udFL_tcShowPanelTab(TC)
	STRUCT WMTabControlAction &TC
	
	switch(TC.eventcode)
		case 2:
			PanelFillTab(TC.tab)
			break
	endswitch
	
	return 0
End

// Buttons

// set paths

Function udFL_bcSetDirPaths(bc) : ButtonControl
	STRUCT WMButtonAction &bc
	
	switch(bc.eventCode)
		case 2:
			PathInfo udFL_DataFiles
			if (!V_flag)
				NewPath/Q udFL_DataFiles, SpecialDirPath("Desktop",0,0,0)
			endif
			PathInfo/S udFL_DataFiles
			ControlInfo rpmode_tb0
			if (V_value==1)
				NewPath/Q/O/M=udFLpan00k udFL_DataFiles
			else
				NewPath/O/M=udFLpan00k udFL_DataFiles			
			endif
			if (V_flag)
				return 0
			endif
			Button setpaths_tb0, title=udFLpan00i2, userdata="1"
			udFL_SubPanelShow(0)
			break
	endswitch
	
	return 0
end

// load or display data

Function udFL_bcDoIt(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	SVAR llowl = root:Packages:StandardizedFileLoader:listofwavesloaded
	SVAR lfn = root:Packages:StandardizedFileLoader:loadfunctionname
	
	variable cm
	
	switch( ba.eventCode )
		case 2:
		
			ControlInfo tb_closeme
			cm = V_Value
					
			strswitch(ba.UserData)
			
				// load
				
				case "Load":
//					ControlInfo fname_tb0
//					lnu = V_value
					ControlInfo loader_tb0
					lfn = StringFromList(V_value-1,udFLLoaderList(1),";")
					llowl = ""
					udFL_LoadData()
					ControlInfo flist_tb0
					TitleBox fldrx_tb0 disable=(cmpstr(S_value,"Create New"))
					break
				case "Display":
					execute/Q "xzDisplayData()"
					break
			endswitch
			if (cm==1)
				DoWindow/K udFL_Panel
			endif
	endswitch

	return 0
End

// Set Variables

// validate new folder string

Function udFL_svcValidateLoadFolderName(sv) : SetVariableControl
	STRUCT WMSetVariableAction &sv

	SVAR lff = root:Packages:StandardizedFileLoader:loadtofolder
	
	variable trn
	string theF
	
	switch(sv.eventCode)
		case 2:
			if (strsearch(sv.sval," ",0)>0)
				lff = "INVALID Spaces!"
				ControlUpdate newdir_tb0
				trn = ticks
				do
				while((ticks-trn)<90)
				lff=""
				SetVariable newdir_tb0 activate
			endif
			theF = "root:" + lff
			TitleBox fldrx_tb0 disable=(!DataFolderExists(theF))
			break	
	endswitch
	
	return 0
	
End

// validate new file string

Function udFL_svcValidateNewTraceName(sv) : SetVariableControl
	STRUCT WMSetVariableAction &sv

	SVAR lfp = root:Packages:StandardizedFileLoader:loadfileprefix
	
	switch(sv.eventCode)
		case 2:
			lfp = CleanupName(lfp,0)
			SetVariable newfname_tb0 activate
			break	
	endswitch
	
	return 0
End

// Checkboxes

// PopupMenu

// change loader modes (and zero the userData) when loader is changed

Function udFL_popChangeLoaderModes(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	SVAR uD = root:Packages:StandardizedFileLoader:userData
	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	
	string lFNa
	
	switch(pa.eventCode)
		case 2:
			lFNa = "udStFiLr" + pa.popStr		
			if (cmpstr(lFNa,lfN)==0)
				return 0
			endif
			uD = ""
			lFN = lfNa		
			PopupMenu lmode_tb0,mode=1,value=#"udFL_FileLoaderModeList()"
			if (strlen(ChildWindowList("#"))!=0)
				udFL_GetFilesInDataDirPath(lFNa)
			endif
	endswitch
	
	return 0
End

// potentially create a new directory name

Function udFL_popCreateNewFolderName(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	SVAR ltf = root:Packages:StandardizedFileLoader:loadtofolder
	
	switch(pa.eventCode)
		case 2:
			TitleBox fldrx_tb0 disable=1
			if (pa.popNum < 3)
				PopupMenu flist_tb0 mode=1
				SetVariable newdir_tb0 disable=0
				ltf = ""
			else
				SetVariable newdir_tb0 disable=1
				ltf = pa.popStr
			endif
	endswitch
	
	return 0
End

// potentially create a new trace name

Function udFL_popCreateNewTraceName(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	
	switch(pa.eventCode)
		case 2:
			SetVariable newfname_tb0 disable=((pa.popNum!=4)&&(pa.popNum!=6))
	endswitch
	
	return 0
End

Function/S udFL_FolderList()

	return (udFLpan00b + ";-;" + udFLFolderList())

end

// List Functions

Function/S udFL_RenameInputList()

	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	
	return (udFLQuerysetNames(lFN))
end


Function/S udFL_FileLoaderModeList()

	SVAR lFN = root:Packages:StandardizedFileLoader:loadfunctionname
	
	return (udFLQueryprocModes(lFN))

End

Function udFL_GetFilesInDataDirPath(lfname)
	string lfname

	wave/T flist = root:Packages:StandardizedFileLoader:filesindatapath
	wave nlist =root:Packages:StandardizedFileLoader:nfilesindatapath

	STRUCT udFLStandardStructure udFL
	udFLInitializeLoader(udFL,lfname)
	
	string mimetype = udFL.mimetype
	string filetypes = udFL.extensions
	
	string lofiles = "", nlof = ""
	variable nfiles, ic, ntypes
	
	lofiles = GrepList(IndexedFile(udFL_DataFiles,-1,"????"),"^\.+",1,";")
	nfiles = ItemsInList(lofiles,";")
	ntypes = ItemsInList(filetypes,";")

	for (ic=0;ic<ntypes;ic+=1)
		nlof += ListMatch(lofiles,("*" + StringFromList(ic,filetypes)))
	endfor
	
	nfiles = ItemsInList(nlof,";")
	redimension/N=(nfiles) flist
	redimension/N=(nfiles) nlist
	flist = StringFromList(p,nlof,";")
	nlist = 0
	
	return 0
end
